@@ -1,13 +1,13 @@
-## drop.tip.R (2017-02-14)
+## drop.tip.R (2017-10-28)
 
 ##   Remove Tips in a Phylogenetic Tree
 
-## Copyright 2003-2017 Emmanuel Paradis
+## Copyright 2003-2017 Emmanuel Paradis, 2017 Klaus Schliep
 
 ## This file is part of the R-package `ape'.
 ## See the file ../COPYING for licensing issues.
 
-extract.clade <- function(phy, node, root.edge = 0, interactive = FALSE)
+extract.clade <- function(phy, node, root.edge = 0, collapse.singles = TRUE, interactive = FALSE)
 {
     n <- length(phy$tip.label)
     if (interactive) {
@@ -29,12 +29,14 @@ extract.clade <- function(phy, node, root.edge = 0, interactive = FALSE)
     }
     if (node == n + 1L) return(phy)
     keep <- prop.part(phy)[[node - n]]
-    drop.tip(phy, (1:n)[-keep], root.edge = root.edge, rooted = TRUE)
+    drop.tip(phy, (1:n)[-keep], root.edge = root.edge, rooted = TRUE,
+             collapse.singles = collapse.singles)
 }
 
 drop.tip <-
     function(phy, tip, trim.internal = TRUE, subtree = FALSE,
-             root.edge = 0, rooted = is.rooted(phy), interactive = FALSE)
+             root.edge = 0, rooted = is.rooted(phy), collapse.singles = TRUE,
+             interactive = FALSE)
 {
     if (!inherits(phy, "phylo"))
         stop('object "phy" is not of class "phylo"')
@@ -95,9 +97,9 @@ drop.tip <-
     if (subtree) {
         trim.internal <- TRUE
         tr <- reorder(phy, "postorder")
-        N <- .C(node_depth, as.integer(Ntip), as.integer(Nnode),
+        N <- .C(node_depth, as.integer(Ntip),
                 as.integer(tr$edge[, 1]), as.integer(tr$edge[, 2]),
-                as.integer(Nedge), double(Ntip + Nnode), 1L)[[6]]
+                as.integer(Nedge), double(Ntip + Nnode), 1L)[[5]]
     }
 
     edge1 <- phy$edge[, 1] # local copies
@@ -169,18 +171,23 @@ drop.tip <-
     ## the tips may not be sorted in increasing order in the
     ## 2nd col of edge, so no need to reorder $tip.label
     phy$edge[TERMS, 2] <- rank(phy$edge[TERMS, 2])
-    phy$tip.label <- phy$tip.label[-tip]
+    ## fix by Thomas Sibley (2017-10-28):
+    if (length(tip)) phy$tip.label <- phy$tip.label[-tip]
 
     ## make new tip labels if necessary:
     if (subtree || !trim.internal) {
         ## get the numbers of the nodes that become tips:
         node2tip <- oldNo.ofNewTips[oldNo.ofNewTips > Ntip]
-        new.tip.label <- if (subtree) {
-            paste("[", N[node2tip], "_tips]", sep = "")
-        } else {
-            if (is.null(phy$node.label)) rep("NA", length(node2tip))
-            else phy$node.label[node2tip - Ntip]
-        }
+        ## fix by Thomas Sibley (2017-10-28):
+        new.tip.label <-
+            if (!length(node2tip)) {
+                character(0)
+            } else if (subtree) {
+                paste("[", N[node2tip], "_tips]", sep = "")
+            } else {
+                if (is.null(phy$node.label)) rep("NA", length(node2tip))
+                else phy$node.label[node2tip - Ntip]
+            }
 #        if (!is.null(phy$node.label))
 #            phy$node.label <- phy$node.label[-(node2tip - Ntip)]
         phy$tip.label <- c(phy$tip.label, new.tip.label)
@@ -199,6 +206,6 @@ drop.tip <-
     storage.mode(phy$edge) <- "integer"
     if (!is.null(phy$node.label)) # update node.label if needed
         phy$node.label <- phy$node.label[which(newNb > 0) - Ntip]
-    collapse.singles(phy)
+    if (collapse.singles) phy <- collapse.singles(phy)
+    phy
 }
-
