@@ -1,8 +1,8 @@
-## rtree.R (2013-04-02)
+## rtree.R (2017-12-18)
 
 ##   Generates Trees
 
-## Copyright 2004-2013 Emmanuel Paradis
+## Copyright 2004-2017 Emmanuel Paradis
 
 ## This file is part of the R-package `ape'.
 ## See the file ../COPYING for licensing issues.
@@ -10,49 +10,50 @@
 rtree <- function(n, rooted = TRUE, tip.label = NULL, br = runif, ...)
 {
     foo <- function(n, pos) {
-        n1 <- sample.int(n - 1, 1, FALSE, NULL)
+        n1 <- sample.int(n - 1L, 1, FALSE, NULL)
         n2 <- n - n1
-        po2 <- pos + 2*n1 - 1
-        edge[c(pos, po2), 1] <<- nod
+        po2 <- pos + 2L * n1 - 1L
+        edge[c(pos, po2), 1L] <<- nod
         nod <<- nod + 1L
-        if (n1 > 2) {
-            edge[pos, 2] <<- nod
-            foo(n1, pos + 1)
-        } else if (n1 == 2) {
-            edge[c(pos + 1, pos + 2), 1] <<- edge[pos, 2] <<- nod
+        if (n1 > 2L) {
+            edge[pos, 2L] <<- nod
+            foo(n1, pos + 1L)
+        } else if (n1 == 2L) {
+            edge[pos + 1:2, 1L] <<- edge[pos, 2L] <<- nod
             nod <<- nod + 1L
         }
-        if (n2 > 2) {
-            edge[po2, 2] <<- nod
-            foo(n2, po2 + 1)
-        } else if (n2 == 2) {
-            edge[c(po2 + 1, po2 + 2), 1] <<- edge[po2, 2] <<- nod
+        if (n2 > 2L) {
+            edge[po2, 2L] <<- nod
+            foo(n2, po2 + 1L)
+        } else if (n2 == 2L) {
+            edge[po2 + 1:2, 1L] <<- edge[po2, 2L] <<- nod
             nod <<- nod + 1L
         }
     }
 
-    if (n < 2) stop("a tree must have at least 2 tips.")
-    nbr <- 2 * n - 3 + rooted
-    edge <- matrix(NA, nbr, 2)
-
     n <- as.integer(n)
-    if (n == 2) {
+
+    if (n < 2) stop("a tree must have at least 2 tips.")
+    nbr <- 2L * n - 3L + rooted
+    edge <- matrix(NA_integer_, nbr, 2L)
+
+    if (n == 2L) {
         if (rooted) edge[] <- c(3L, 3L, 1L, 2L)
         else stop("an unrooted tree must have at least 3 tips.")
-    } else if (n == 3) {
+    } else if (n == 3L) {
         edge[] <-
           if (rooted) c(4L, 5L, 5L, 4L, 5L, 1:3)
           else c(4L, 4L, 4L, 1:3)
-    } else if (n == 4 && !rooted) {
+    } else if (n == 4L && !rooted) {
         edge[] <- c(5L, 6L, 6L, 5L, 5L, 6L, 1:4)
     } else {
         nod <- n + 1L
         if (rooted) { # n > 3
-            foo(n, 1)
+            foo(n, 1L)
             ## The following is slightly more efficient than affecting the
             ## tip numbers in foo(): the gain is 0.006 s for n = 1000.
-            i <- which(is.na(edge[, 2]))
-            edge[i, 2] <- 1:n
+            i <- which(is.na(edge[, 2L]))
+            edge[i, 2L] <- 1:n
         } else { # n > 4
             n1 <- sample.int(n - 2L, 1L)
             if (n1 == n - 2L) {
@@ -63,7 +64,7 @@ rtree <- function(n, rooted = TRUE, tip.label = NULL, br = runif, ...)
             }
             po2 <- 2L * n1
             po3 <- 2L * (n1 + n2) - 1L
-            edge[c(1, po2, po3), 1L] <- nod
+            edge[c(1L, po2, po3), 1L] <- nod
             nod <- nod + 1L
             if (n1 > 2L) {
                 edge[1L, 2L] <- nod
@@ -79,15 +80,15 @@ rtree <- function(n, rooted = TRUE, tip.label = NULL, br = runif, ...)
                 edge[c(po2 + 1L, po2 + 2), 1L] <- edge[po2, 2L] <- nod
                 nod <- nod + 1L
             }
-            if (n3 > 2) {
+            if (n3 > 2L) {
                 edge[po3, 2L] <- nod
                 foo(n3, po3 + 1L)
             } else if (n3 == 2L) {
-                edge[c(po3 + 1L, po3 + 2), 1L] <- edge[po3, 2L] <- nod
+                edge[c(po3 + 1L, po3 + 2L), 1L] <- edge[po3, 2L] <- nod
                 ## nod <- nod + 1L
             }
             i <- which(is.na(edge[, 2L]))
-            edge[i, 2] <- 1:n
+            edge[i, 2L] <- 1:n
         }
     }
     phy <- list(edge = edge)
@@ -98,7 +99,7 @@ rtree <- function(n, rooted = TRUE, tip.label = NULL, br = runif, ...)
         phy$edge.length <-
             if (is.function(br)) br(nbr, ...) else rep(br, length.out = nbr)
     }
-    phy$Nnode <- n - 2L + rooted
+    phy$Nnode <- n - 2L + as.integer(rooted)
     class(phy) <- "phylo"
     attr(phy, "order") <- "cladewise"
     phy
@@ -120,7 +121,7 @@ rcoal <- function(n, tip.label = NULL, br = "coalescent", ...)
         edge.length <- c(x[c(2, 1, 1)], sum(x))
     } else {
         edge.length <- numeric(nbr)
-        h <- numeric(2*n - 1) # initialized with 0's
+        h <- numeric(2*n - 1)
         node.height <- cumsum(x)
         pool <- 1:n
         nextnode <- 2L*n - 1L
